/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// About.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "About.h"
#include "customization.h"
#include "pim/ClientSettings.h"
#include "ClientUtil.h"
#include "LogLevelSettings.h"
#include "DeviceInfo.h"

#if defined(WIN32_PLATFORM_WFSP)
#define VPADDING 5
#else
#define VPADDING 17
#endif


// CAbout dialog

IMPLEMENT_DYNAMIC(CAbout, CDialog)

CAbout::CAbout(CWnd* pParent /*=NULL*/)
	: CDialog(CAbout::IDD, pParent)
{
    
}


CAbout::~CAbout() {}

void CAbout::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_ABOUT_MSG, staticAboutMsg);
}


BEGIN_MESSAGE_MAP(CAbout, CDialog)
    ON_WM_SIZE()

    ON_WM_CTLCOLOR()
    ON_WM_VSCROLL()
    ON_COMMAND(ID_MENU_DEVICEINFO, &CAbout::OnDeviceInfo)
    ON_COMMAND(ID_MENU_LOGGING, &CAbout::OnMenuLogging)
END_MESSAGE_MAP()

void CAbout::OnDeviceInfo(){

    CDeviceInfo wnd;
    INT_PTR result = wnd.DoModal();
}

void CAbout::OnMenuLogging(){

    CLoglevelSettings wnd;
    INT_PTR result = wnd.DoModal();
    if (result == 5) {
        EndDialog(0);
    }
}

#if defined (WIN32_PLATFORM_PSPC)
BOOL CAbout::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if( ( (pMsg->message == WM_KEYDOWN)|| (pMsg->message == WM_KEYUP))){
        WORD wScrollNotify = 0xFFFF; 
        if ( resIdOld == IDD_UI_ABOUT_WIDE ){
            if(pMsg->wParam == VK_RIGHT){
                wScrollNotify = SB_LINEDOWN;
            }else if (pMsg->wParam == VK_LEFT){
                wScrollNotify = SB_LINEUP;
            }
        }else{
            if(pMsg->wParam == VK_DOWN){
                wScrollNotify = SB_LINEDOWN;
            }else if (pMsg->wParam == VK_UP){
                wScrollNotify = SB_LINEUP;
            }
        }


        ::SendMessage(GetSafeHwnd(), WM_VSCROLL, MAKELONG(wScrollNotify, 0), 0L);
    };

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}
#endif

/**
* Methods of About form
*/

void CAbout::initFontForLicense() {

    // font for the GPL license
    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    // cr font
    LOGFONT cpf;
    memset(&cpf, 0, sizeof(LOGFONT));
    HFONT cpFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(cpFont, sizeof cpf, &cpf);
    LONG h = lf.lfHeight;
    h = h + 2;
    lf.lfHeight = h ;
    VERIFY(crFont.CreateFontIndirect(&lf));
}



BOOL CAbout::OnInitDialog()
{
    CString s1;

    // set window text title
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT); SetWindowText(s1);

    CDialog::OnInitDialog();

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_SK1_CLOSE;
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    
    if(!SHCreateMenuBar(&dlgCommandBar)){
      TRACE(_T("Cannot create command bar!"));
      return FALSE;
    };
        
    // Set all scrollbar parameters
    GetWindowRect(&dlgRect);    
    SCROLLINFO si;
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_ALL;                 // SIF_ALL = SIF_RANGE | SIF_PAGE | SIF_POS | SIF_TRACKPOS;
    si.nMin = 0;
    si.nMax = dlgMaxHeight;
    si.nPage = dlgRect.Height();
    si.nPos = 0;
    SetScrollInfo(SB_VERT, &si, TRUE);

    // init variables.
    s1              = "";
    aboutMessage    = "";
    versionMessage  = "";    
    HKEY key;
    long err        = 0;
    ULONG dim;
    TCHAR *buf      = NULL;

    // load the version from the registry
    RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("\\Software\\") ROOT_CONTEXT_W,
        0, KEY_READ,  &key);
    if(key != 0)
    {
        // Get value length
        err = RegQueryValueEx(key, PLUGIN_VERSION, NULL, NULL, NULL, &dim);

        if (err == ERROR_SUCCESS) {
            if (dim > 0) {
                buf = new TCHAR[dim + 1];
                err = RegQueryValueEx(key, PLUGIN_VERSION, NULL, NULL,(UCHAR*)buf, &dim);
            }
        }
    }
    RegCloseKey(key);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT_MSG1);
    s1 = escapeAmpString(s1);
    
    CString s2(buf);
    if(s2 != ""){
        s1 += s2;
        s1 += "\n\n";
    }
    
    if(buf){
        delete [] buf; buf = NULL;
    }
    
    // add the Version number
    aboutMessage += s1;

    //load info about the ip, device info. Not to be showd
    if(FUNOPT_SHOW_DEV_INFO){

        WCHAR wszMachineName[128];
        SystemParametersInfo(SPI_GETOEMINFO, sizeof(wszMachineName),
            &wszMachineName, 0);
        WCHAR tst[500];
        wsprintf(tst,L"%s",wszMachineName);
        s1 ="";
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DEVICE);
        s1 += ": ";
        s1 += tst;
        s1 += "\n";
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_DEVID);
        s1 += s2;
        s1 += " ";
        s1 += getRegConfig()->getDeviceConfig().getDevID();
        s1 += "\n";
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_CURRIP);
        s1 += s2;
        s1 += " ";
        s1 += getRegConfig()->getIp().c_str();
        s1 += "\n";

        aboutMessage += s1;
    }

    // load copyright information of the company from the resource
    // "Copyright  2003 - 2009 Funambol, Inc.\nAll rights reserved."
    if(FUNOPT_SHOW_COPYRIGHT){
        versionMessage += "\n";
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT_MSG2);
        versionMessage += s2;
    }
    // Adds: "\nwww.funambol.com"
    if (FUNOPT_SHOW_ABOUT_MAIN_WEB_SITE) {
        versionMessage += "\n\n";
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT_MAIN_WEB_SITE);
        versionMessage += s2;
    }

    // init the font for the GPL license and popuplate crFont
    initFontForLicense();    

    if(FUNOPT_SHOW_COPYRIGHT){
#if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT); SetDlgItemText(IDC_HEADER_STATIC, s1);
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif
    }
    
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return TRUE;
}


void CAbout::OnSize(UINT nType, int cx, int cy)
{

    CDialog::OnSize(nType, cx, cy);
#if defined(WIN32_PLATFORM_WFSP)
        ::SendMessage(GetSafeHwnd(), DM_RESETSCROLL, TRUE, TRUE);
#endif

    CString s1;
#if defined(_DEVICE_RESOLUTION_AWARE)&& !defined(WIN32_PLATFORM_WFSP)
    int resId;
    switch(DRA::GetDisplayMode())
    {
        case DRA::Landscape:
            resId = IDD_UI_ABOUT_WIDE; break;
        case DRA::Portrait:
            resId = IDD_UI_ABOUT; break;
        case DRA::Square:
            resId = IDD_UI_ABOUT_SQUARE; break;
        default:
            resId = IDD_UI_ABOUT; break;
    };

    DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
    resIdOld = resId;

#endif

    // set image position
    CRect rectImage;
    CRect rectClient;
    int x = 0;
    GetClientRect(&rectClient);
    GetDlgItem(IDC_IMG_FUNLOGO)->GetClientRect(&rectImage);
    x = (rectClient.Width()/2) - (rectImage.Width()/2);
    GetDlgItem(IDC_IMG_FUNLOGO)->SetWindowPos(&CWnd::wndTop, x,VPADDING,0,0,SWP_NOSIZE);

    // set text position
    CRect rectText;
    CRect rectTextVersion;
    GetClientRect(&rectClient);
    GetDlgItem(IDC_ABOUT_MSG)->GetClientRect(&rectText);
    GetDlgItem(IDC_ABOUT_MSG2)->GetClientRect(&rectTextVersion);
    x = (rectClient.Width()/2) - (rectText.Width()/2);
    
    if (FUNOPT_SHOW_DEV_INFO){
        modifyWindow(GetDlgItem(IDC_ABOUT_MSG), 0, -130, 0 , 100);
    }
    else{
        GetDlgItem(IDC_ABOUT_MSG)->SetWindowPos(&CWnd::wndTop, x,rectImage.BottomRight().y + 15,0,0,SWP_NOSIZE);
        GetDlgItem(IDC_ABOUT_MSG2)->SetWindowPos(&CWnd::wndTop, x,rectImage.BottomRight().y + rectText.BottomRight().y + 10,0,0,SWP_NOSIZE);
    }

    SetDlgItemText(IDC_ABOUT_MSG, aboutMessage);
    SetDlgItemText(IDC_ABOUT_MSG2, versionMessage);

    s1 = "";
    
    if (FUNOPT_SHOW_LICENSE) {
        s1.Append(TEXT("This program is provided AS IS, without warranty licensed under AGPLV3. The Program is free software; you can redistribute it and/or modify it under the terms of the GNU Affero General Public License version 3 as published by the Free Software Foundation including the additional permission set forth source code file header.\n\n"));
        s1.Append(TEXT("The interactive user interfaces in modified source and object code versions of this program must display Appropriate Legal Notices, as required under Section 5 of the GNU Affero General Public License version 3.\n\n"));
        s1.Append(TEXT("In accordance with Section 7(b) of the GNU Affero General Public License version 3, these Appropriate Legal Notices must retain the display of the \"Powered by Funambol\" logo. If the display of the logo is not reasonably feasible for technical reasons, the Appropriate Legal Notices must display the words \"Powered by Funambol\". Funambol is a trademark of Funambol, Inc."));    
        SetDlgItemText(IDC_ABOUT_CR, s1);
        GetDlgItem(IDC_ABOUT_CR)->SetFont(&crFont);
        GetDlgItem(IDC_POWERED_FUN)->ShowWindow(SW_HIDE);

    } else {
        // empty the license
        GetDlgItem(IDC_ABOUT_CR)->ShowWindow(SW_HIDE);
        CRect licenseRec;                
        GetDlgItem(IDC_ABOUT_CR)->GetClientRect(&licenseRec);
        licenseRec.SetRectEmpty();
        
        // the powered by funambol image has fixed dimension 24 x 128
        int poweredX = 128;
        int poweredY = 24;
        GetDlgItem(IDC_POWERED_FUN)->ShowWindow(SW_SHOW); 
        GetClientRect(&rectClient);        
        
        x = (rectClient.Width()/2) - (poweredX/2);
        // move little up the message box 2
        GetDlgItem(IDC_POWERED_FUN)->SetWindowPos(&CWnd::wndTop, x, rectImage.BottomRight().y 
                                                                  + rectText.BottomRight().y 
                                                                  + rectTextVersion.BottomRight().y + 15, poweredX, poweredY, SWP_SHOWWINDOW);        
                        
    }

    // set fonts
    GetDlgItem(IDC_ABOUT_MSG)->SetFont(&fontBold);
    
#if defined(WIN32_PLATFORM_WFSP)
        GetDlgItem(IDC_ABOUT_CR)->SetWindowPos(&CWnd::wndTop, x, 
            (rectImage.BottomRight().y + rectText.BottomRight().y + 
            rectTextVersion.BottomRight().y + 35),
        0,0,SWP_NOSIZE);
#else
        GetDlgItem(IDC_ABOUT_CR)->SetWindowPos(&CWnd::wndTop, 3, 
            (rectImage.BottomRight().y + rectText.BottomRight().y +
            rectTextVersion.BottomRight().y + 35),
        0,0,SWP_NOSIZE);
#endif

    
    // Manage the vertical scrolling-bar
    //
    GetWindowRect(&dlgRect);
    
    dlgCurrentHeight = cy;    
    scrollbarPosition = 0; // reset bar position

    // Get the position of the bottom object of the dialog,
    // the max scrollbar height is derived from this.
    // *** NOTE: CHANGE IT IF THE OBJECT POSITION CHANGES! ***    
    if (FUNOPT_SHOW_LICENSE) {
        CRect aboutmessageRect;
        GetDlgItem(IDC_ABOUT_CR)->GetClientRect(&aboutmessageRect);
        dlgMaxHeight = rectImage.bottom + rectText.bottom + rectTextVersion.bottom + aboutmessageRect.bottom + 45;
    } else {
        CRect poweredText;        
        GetDlgItem(IDC_POWERED_FUN)->GetClientRect(&poweredText);
        dlgMaxHeight = rectImage.bottom + rectText.bottom + rectTextVersion.bottom + poweredText.bottom + 20;
        
    }

    SCROLLINFO si;
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_PAGE;                // Set only the new nPage = cy
    si.nPage = cy;
    //si.nPage = dlgRect.Height();
    si.nPos = 0;
    si.nMax = dlgMaxHeight;
    SetScrollInfo(SB_VERT, &si, TRUE);


#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT); SetDlgItemText(IDC_HEADER_STATIC, s1);
#endif

}

HBRUSH CAbout::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif
    if(pWnd->GetDlgCtrlID() == IDC_ABOUT_CR){
        COLORREF grey = RGB(180,180,180);        
        pDC->SetTextColor(grey);
    }
    

    return hbr;
}

/**
 * Called when vertical scrollbar is touched.
 * Manages the window scrolling.
 */
void CAbout::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
    int nDelta;
    int nMaxPos = dlgMaxHeight - dlgCurrentHeight;

    // Calculate the offset to scroll (nDelta)
    switch (nSBCode) 
    {
        case SB_LINEDOWN:
            if (scrollbarPosition >= nMaxPos)
                return;
            nDelta = 2;
            break;

        case SB_LINEUP:
            if (scrollbarPosition <= 0)
                return;
            nDelta = -2;
            break;

        case SB_THUMBPOSITION:
            nDelta = (int)nPos - scrollbarPosition;
            break;
        case SB_PAGEDOWN:        
            if (scrollbarPosition >= nMaxPos)
                return;            
            nDelta = min(nMaxPos/10, nMaxPos - scrollbarPosition);
            break;                           
        case SB_PAGEUP:
            if (scrollbarPosition <= 0)
                return;
            nDelta = -min(nMaxPos/10, scrollbarPosition);
            break;
         case SB_THUMBTRACK:                
            if (LOBYTE(nSBCode) == SB_THUMBTRACK) {                           
                nDelta = nPos - scrollbarPosition;
            }
            break;
        default:
            // Don't handle other commands...
            return;
    }

    // Move the scrollbar
    scrollbarPosition += nDelta;
    SetScrollPos(SB_VERT, scrollbarPosition, TRUE);

    // Scroll the dialog window
    ScrollWindowEx(0, -nDelta, NULL, &dlgRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
    
    CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CAbout::OnAppLink(){
    SHELLEXECUTEINFO lpExecInfo;
    memset(&lpExecInfo, 0, sizeof(SHELLEXECUTEINFO));
    lpExecInfo.cbSize = sizeof(SHELLEXECUTEINFO);

    CString site("http://");
    site += APP_LINK;
    lpExecInfo.lpFile = site;
    lpExecInfo.nShow = SW_SHOWNORMAL;
    lpExecInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
    lpExecInfo.lpVerb = _T("open");
    ShellExecuteEx(&lpExecInfo);
    ZeroMemory(&lpExecInfo, sizeof(SHELLEXECUTEINFO));
}
